/*
 * Startup.S : processes after CPU RESET
 *   by uratan! 2020.6.24
 */

/*
 * specify RAM address range, I will fill them at startup
 */
#define FILL_RAM_TOP  0x10000000
#define FILL_RAM_END  0x10002000	/* SRAM0 8kB */

/*===================================================================*/

/*
 * Stack Configuration (Stack Sizes in Bytes)
 */
	.set Stack_Size, 0x00000400

	.equ Stack_Size, Stack_Size & ~3		/* regulate */

.thumb
.section .stack, "w"
.align 4
	.space	Stack_Size
Top_Stack:


/*===================================================================*/

.text

/*
 * Exception Vectors, Mapped to Address 0.
 *   VECTOR ADDRESSES SHALL BE ODD !  (there may be a directive for it...)
 */
.thumb
.section .vectorg, "ax"

	.extern	def_NMI_Handler
	.extern	def_HardFault_Handler
	.extern	def_Exception_Handler
	.extern	def_SVCall_Handler
	.extern	def_PendSV_Handler
	.extern	SysTick_Handler
	.extern	USART_IRQ_Handler
	.extern	PIN_INT0_IRQ_Handler
	.extern	USB_IRQ_Handler

__Vectors:
	.word	Top_Stack					/*  0: Initial SP     */
	.word	1+ Reset_Handler			/*  1: Reset          */
	.word	1+ def_NMI_Handler			/*  2: NMI            */
	.word	1+ def_HardFault_Handler	/*  3: HardFault      */
	.word	1+ def_Exception_Handler	/*  4: Reserved       */
	.word	1+ def_Exception_Handler	/*  5: Reserved       */
	.word	1+ def_Exception_Handler	/*  6: Reserved       */
	.word	0x00000000					/*  7: Reserved (boot signature) */
	.word	1+ def_Exception_Handler	/*  8: Reserved       */
	.word	1+ def_Exception_Handler	/*  9: Reserved       */
	.word	1+ def_Exception_Handler	/* 10: Reserved       */
	.word	1+ def_SVCall_Handler		/* 11: SVCall         */
	.word	1+ def_Exception_Handler	/* 12: Reserved       */
	.word	1+ def_Exception_Handler	/* 13: Reserved       */
	.word	1+ def_PendSV_Handler		/* 14: PendSV         */
	.word	1+ SysTick_Handler			/* 15: SysTick        */
		/* IRQs are continued... */
	.word	1+ PIN_INT0_IRQ_Handler		/* 16: (0) PIN_INT0_IRQ */
	.word	1+ def_Exception_Handler	/* 17: (1) PIN_INT1_IRQ */
	.word	1+ def_Exception_Handler	/* 18: (2) PIN_INT2_IRQ */
	.word	1+ def_Exception_Handler	/* 19: (3) PIN_INT3_IRQ */
	.word	1+ def_Exception_Handler	/* 20: (4) PIN_INT4_IRQ */
	.word	1+ def_Exception_Handler	/* 21: (5) PIN_INT5_IRQ */
	.word	1+ def_Exception_Handler	/* 22: (6) PIN_INT6_IRQ */
	.word	1+ def_Exception_Handler	/* 23: (7) PIN_INT7_IRQ */
	.word	1+ def_Exception_Handler	/* 24: (8) GINT0_IRQ    */
	.word	1+ def_Exception_Handler	/* 25: (9) GINT1_IRQ    */
	.word	1+ def_Exception_Handler	/* 26:(10)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 27:(11)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 28:(12)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 29:(13)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 30:(14) SSP1_IRQ     */
	.word	1+ def_Exception_Handler	/* 31:(15) I2C_IRQ      */
	.word	1+ def_Exception_Handler	/* 32:(16) CT16B0_IRQ   */
	.word	1+ def_Exception_Handler	/* 33:(17) CT16B1_IRQ   */
	.word	1+ def_Exception_Handler	/* 34:(18) CT32B0_IRQ   */
	.word	1+ def_Exception_Handler	/* 35:(19) CT32B1_IRQ   */
	.word	1+ def_Exception_Handler	/* 36:(20) SSP0_IRQ     */
	.word	1+ USART_IRQ_Handler		/* 37:(21) USART_IRQ    */
	.word	1+ USB_IRQ_Handler			/* 38:(22) USB_IRQ      */
	.word	1+ def_Exception_Handler	/* 39:(23) USB_FIQ      */
	.word	1+ def_Exception_Handler	/* 40:(24) ADC_IRQ      */
	.word	1+ def_Exception_Handler	/* 41:(25) WWDT_IRQ     */
	.word	1+ def_Exception_Handler	/* 42:(26) BOD_IRQ      */
	.word	1+ def_Exception_Handler	/* 43:(27) FLASH_IRQ    */
	.word	1+ def_Exception_Handler	/* 44:(28)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 45:(29)  (rsrv)      */
	.word	1+ def_Exception_Handler	/* 46:(30) USB_WAKEUP_IRQ */
	.word	1+ def_Exception_Handler	/* 47:(31) IOH_IRQ      */


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

/*
 * system startup processes
 */

.thumb
.section .init, "ax"
.global __startup
.func __startup
__startup:

Reset_Handler:
	/*
	 * fill all memory for debug
	 */
	LDR		R0, =0x77777777
	LDR		R1, =FILL_RAM_TOP
	LDR		R2, =FILL_RAM_END
lp01:
	CMP		R1, R2
	BHS		lp02
	STR		R0, [R1]
	ADD		R1, #4
	B		lp01
lp02:

	/*
	 * marking the stack area
	 */
	LDR		R3, =Top_Stack
	MOV		R2, R3
	LDR		R1, =Stack_Size
	SUB		R2, R1
	LDR		R1, =0xAAAAAAAA		/* mAin stack */
lp11:
	CMP		R2, R3
	BHS		lp12
	STR		R1, [R2]
	ADD		R2, #4
	B		lp11
lp12:

	/*
	 * Relocate .data section (Copy from ROM to RAM)
	 */
	LDR		R1, =_text_end
	LDR		R2, =_data_start
	LDR		R3, =_data_end
lp21:
	CMP		R2, R3
	BHS		lp22
	LDR		R0, [R1]
	STR		R0, [R2]
	ADD		R1, #4
	ADD		R2, #4
	B		lp21
lp22:

	/*
	 * Clear .bss section (Zero init)
	 */
	MOV		R0, #0
	LDR		R1, =_bss_start
	LDR		R2, =_bss_end
lp31:
	CMP		R1, R2
	BHS		lp32
	STR		R0, [R1]
	ADD		R1, #4
	B		lp31
lp32:

	/*
	 * Enter the C code
	 */
	LDR		R0,=main
	MOV		PC,R0
.endfunc

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

/*
 * firmware update processes
 */

	.set EXEC_BUFF_SIZE, 0x0400

.thumb
.text
.section .text, "ax"
.global copy_n_exec_on_ram
.func   copy_n_exec_on_ram
copy_n_exec_on_ram:
	/*
	 * copy codes for update to RAM
	 */
	LDR		R1, =_fup_start
	LDR		R2, =_fup_end
	LDR		R3, =exec_buffer
	SUB		R0, R1
	ADD		R0, R3
lp51:
	CMP		R1, R2
	BHS		lp52
	LDR		R4, [R1]
	STR		R4, [R3]
	ADD		R1, #4
	ADD		R3, #4
	B		lp51
lp52:

	/*
	 * execute it
	 */
	MOV		PC, R0

.endfunc

.section .exec, "w"
.align 4
exec_buffer:
	.space	EXEC_BUFF_SIZE


.end
