/*
 * common.h - definitions used system-wide
 *   by uratan! 2020.6.22
 */
#ifndef __COMMON_H__
#define __COMMON_H__

/* =============================================================== */

/*
 * system-wide macros and typedefs
 */
#define NULL    ((void *)0)

// typedef-ed in lpc_types.h
//#define TRUE    1
//#define FALSE   0

#define DONE    0
#define YET     1

//typedef unsigned char  BYTE;
//typedef unsigned short WORD;
//typedef unsigned long  DWORD;
typedef unsigned int   BOOL;

typedef signed char    s_char;
typedef unsigned char  u_char;
typedef unsigned short u_short;
typedef unsigned long  u_long;

typedef void (*INTR_subFUNCs)(u_long x);

#define VOLATILE  volatile		/* enhance to look attractive */

/* =============================================================== */

/*
 * use '?' as HEAD_RESET, wakeup by ISPen for SOLO development
 */
#define DEVELOP_by_SOLO  0

/*
 * work around for "EP0_OUT can't STALL" problem
 */
#define WA_X_STALL_EP0_OUT   1

/* =============================================================== */

/*
 * for software timer (loop by VOLATILE u_long)
 */
#define LOOP_WAIT_1msec  3203		/* 1.0msec @ 48MHz, FLASHTIM=2, -Os */
//#define LOOP_WAIT_1msec  1853		/* 1.0msec @ 24MHz, FLASHTIM=1, -Os */
//#define LOOP_WAIT_1msec  1092		/* 1.0msec @ 12MHz, FLASHTIM=0, -Os */

/*
 * interrupt priorities
 *   0(highest) - 3(lowest)
 */
#define PRI_SYSTICK     1
#define PRI_UART0       1		/* shall be stronger than PRI_USB if useROMdriver */
#define PRI_USB         2
#define PRI_PIN_INT0   PRI_USB		/* use same for wakeup reason detection */
#define PRI_USB_WAKEUP PRI_USB

/*
 * I/O pin usage (AE-LPC11U35-MB)
 *   PIO0_0  (input)  - push-SW2 (xRESET, xLED2-red)
 *   PIO0_1  (input)  - push-SW1 (xISP-EN)
 *   PIO0_3  (input)  - VBUS (ISP-mode: USB/xUART)
 *   PIO0_6  (output) - xUSB-CONNECT (xLED1-red)
 *   PIO0_7  (output) - LED3-red (user)
 * my usage
 *   PIO0_9  (output) - MOSI0 (use as debug-out)
 *   PIO0_18 (input)  - USART-RxD
 *   PIO0_19 (output) - USART-TxD
 *   PIO0_21 (output) - MOSI1 (use as debug-uart)
 */
#define GPI_pSW    (1<<1)		/* SW1/ISP-EN on PIO0_1 */
#define GPO_LED    (1<<7)		/*       LED3 on PIO0_7 */
#define GPO_DEBUG  (1<<9)		/* debug out  on PIO0_9 */
	/* shall rename to distinguish if use pio1_n */

/*
 * use LED3-red as monitor
 */
#define MON_LED_ON    LPC_GPIO->SET[0] = GPO_LED
#define MON_LED_OFF   LPC_GPIO->CLR[0] = GPO_LED
#define MON_LED_TGL   LPC_GPIO->NOT[0] = GPO_LED

/*
 * use this for debug
 */
#define SET_DEBUG_PORT   LPC_GPIO->SET[0] = GPO_DEBUG
#define CLR_DEBUG_PORT   LPC_GPIO->CLR[0] = GPO_DEBUG
#define TGL_DEBUG_PORT   LPC_GPIO->NOT[0] = GPO_DEBUG

/*
 * which SSP to use fro DEBUG-uart ?
 */
#define USE_SSPn     1			/* 0 or 1 */

#if USE_SSPn == 0
#  define          LPC_SSPn                 LPC_SSP0
#  define        RESET_SSPn               RESET_SSP0
#  define SYSCTL_CLOCK_SSPn        SYSCTL_CLOCK_SSP0
#  define              SSPnCLKDIV               SSP0CLKDIV
#else /* USE_SSPn == 0 */
#  define          LPC_SSPn                 LPC_SSP1
#  define        RESET_SSPn               RESET_SSP1
#  define SYSCTL_CLOCK_SSPn        SYSCTL_CLOCK_SSP1
#  define              SSPnCLKDIV               SSP1CLKDIV
#endif /* USE_SSPn == 0 */

/* =============================================================== */

/*
 * shared functions
 */
	/* in common.c */
extern u_long IntDisable(void);
extern void IntRestore(u_long imf);
extern void cm_ssp_uart_init(u_long baudrate, u_long mainCLK);
extern void cm_putchar(int ch);
extern int is_hex(u_char c);
extern u_long to_bin(u_char c);
	/* in rprintf.c */
extern void rprintf(char const *format, ...);

/*
 * shared variables
 */
#ifdef __MAIN_C__
# define EXTERN
#else /* __MAIN_C__ */
# define EXTERN extern
#endif /* __MAIN_C__ */

//@@//#define SIZE_of_L_BUFF    256
//@@//EXTERN u_char l_buff[SIZE_of_L_BUFF];	/* for console line buffer */

#undef EXTERN


#endif /* __COMMON_H__ */
