/*
 * link.cmd : Linker Script File, from winARM example 'LPC2138-ROM.ld'
 *   by uratan! 2020.8.26
 */
	/*
	 * LPC11U35FHI33/501 memories
	 *
	 * Boot-ROM
	 *   0x1FFF_0000--0x1FFF_3FFF (16kB)
	 *
	 * on-chip-flash
	 *   0x0000_0000--0x0000_FFFF (64kB)
	 *    (CRP: 0x0000_02FC)          (was 0x01FC @ LPC214x)
	 *
	 * RAM
	 *   0x1000_0000--0x1000_1FFF (8kB SRAM0)
	 *   0x2000_0000--0x2000_07FF (2kB SRAM1)
	 *  (0x2000_4000--0x2000_47FF (2KB USB RAM)
	 */

/*
 * Memory Definitions
 */
MEMORY
{
	ROM (rx) : ORIGIN = 0x00000000, LENGTH = 16k		/* flash, limit by fup size */
	RAM (rw) : ORIGIN = 0x10000000, LENGTH = 8k - 32	/* avoid IAP work */
}

/*
 * Section Definitions
 */
SECTIONS
{
	/* first section is .text which is used for code */
	.text :
	{
		KEEP(*(.vectorg))
		. = ALIGN(4);
		KEEP(*(.init))			/* Startup code from .init-section */
		*(.text)				/* remaining code */
		*(.text.*)
		. = ALIGN(4);
		_fup_start = . ;
		*(.fup)
		. = ALIGN(4);
		_fup_end = . ;
		*(.rodata)				/* read-only data (constants) */
		*(.rodata.*)
	} > ROM
	. = ALIGN(4);
	_text_end = . ;

	/* .data section which is used for initialized data */
	.data : AT (_text_end)
	{
		_data_start = .;
		*(.data)
		*(.data.*)
	} > RAM
	. = ALIGN(4);
	_data_end = . ;

	/* .bss section which is used for uninitialized data */
	.bss (NOLOAD) :
	{
		_bss_start = . ;
		*(.bss)
		*(COMMON)
	} > RAM
	. = ALIGN(4);
	_bss_end = . ;

	.stack ALIGN(256) :
	{
		*(.stack)
	} > RAM

	.exec ALIGN(4) :
	{
		*(.exec)
	} > RAM

	_end = . ;
}
