#
# sign.awk - gen boot signature from HEX file
#
# +-----------------------------------------------
# |:10000000F0050010C1000000ED040000FD04000038
# |:10001000DD040000DD040000DD040000000000003D
# +---------------------------------********------
#                        |  |  |      expect filled by zero
#                        V  V  V
# +----------------------------------------------------
# |:10 0000 00 F0050010 C1000000 ED040000 FD040000 38
# |:10 0010 00 DD040000 DD040000 DD040000 00000000 3D
# +------------------------------------------------**--
#                                         |  |  |  |
#                                         V  V  V  V
# +----------------------------------------------------
# |:10 0010 00 DD040000 DD040000 DD040000 CEE0FFEF A1
# +----------------------------------------------------
#

BEGIN {
	hex_tab = "0123456789ABCDEF";
	sig = 0;
}

/^:10000000/ {
	#
	# get first half of vector area
	#
	vectors(1);
	next;
}

/^:10001000/ {
	#
	# get second half of vector area
	#
	vectors(2);
	next;
}

{
	#
	# pass thru
	#
	print $0;
}


#
# parse vector area
#
function vectors(n_record)
{
	#
	# split the line to 2digit hexs
	#
	idx = 0;
	lll = length($0);
	for(i=2; i<=lll; i+=2) {				# skip ':'
		c = substr($0, i, 2);
		bin[idx] = hex2bin(c);
		idx++;
	}

#	for(i=0; i<idx; i++) {
#		printf(" %02x", bin[i]);
#	}
#	printf("\n");

	#
	# bind each vector and sum them up (little endian)
	#
	sig += bin[ 7] *0x1000000 + bin[ 6] *0x10000 + bin[ 5] *0x100 + bin[ 4];
	sig += bin[11] *0x1000000 + bin[10] *0x10000 + bin[ 9] *0x100 + bin[ 8];
	sig += bin[15] *0x1000000 + bin[14] *0x10000 + bin[13] *0x100 + bin[12];
	sig += bin[19] *0x1000000 + bin[18] *0x10000 + bin[17] *0x100 + bin[16];

	if(n_record==1) {
		#
		# pass thru
		#
		print $0;
	}
	else {
		#
		# make signature string
		#
		sig = 0 - sig;

		hh = sprintf("%08X", sig);
		sigHH = hex2bin(substr(hh, 1, 2));
		sigHL = hex2bin(substr(hh, 3, 2));
		sigLH = hex2bin(substr(hh, 5, 2));
		sigLL = hex2bin(substr(hh, 7, 2));

		sig = sprintf("%02X%02X%02X%02X", sigLL, sigLH, sigHL, sigHH);

		#
		# adjust check-sum
		#
		sum = bin[20];
		sum -= (sigLL + sigLH + sigHL + sigHH);
		sum += 0x400;
		sum %= 0x100;
		sum = sprintf("%02X", sum);

		#
		# replace signature and the check-sum
		#
		left = substr($0, 1, 33);
		new = left sig sum;
		print new;
	}
}

#
# convert 2digit hex string to bin
#
function hex2bin(h2b_ii)
{
	h2b_c = substr(h2b_ii, 1, 1);
	h2b_b1 = match(hex_tab, h2b_c) - 1;

	h2b_c = substr(h2b_ii, 2, 1);
	h2b_b2 = match(hex_tab, h2b_c) - 1;

	return (h2b_b1 * 16 + h2b_b2);
}

